# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import Optional, Union

from bungio.models.base import ClientMixin, custom_define
from bungio.models.auth import AuthData

from bungio.models import DestinyFireteamFinderGetPlayerApplicationsResponse
from bungio.models import DestinyFireteamFinderGetCharacterActivityAccessResponse
from bungio.models import DestinyFireteamFinderRespondToApplicationResponse
from bungio.models import DestinyFireteamFinderRespondToAuthenticationRequest
from bungio.models import DestinyFireteamFinderListing
from bungio.models import DestinyFireteamFinderOffer
from bungio.models import DestinyFireteamFinderApplicationType
from bungio.models import DestinyFireteamFinderGetPlayerOffersResponse
from bungio.models import DestinyFireteamFinderLobbyResponse
from bungio.models import DestinyFireteamFinderBulkGetListingStatusResponse
from bungio.models import DestinyFireteamFinderUpdateLobbySettingsRequest
from bungio.models import DestinyFireteamFinderHostLobbyResponse
from bungio.models import DestinyFireteamFinderSearchListingsByClanResponse
from bungio.models import DestinyFireteamFinderRespondToApplicationRequest
from bungio.models import DestinyFireteamFinderGetListingApplicationsResponse
from bungio.models import DestinyFireteamFinderRespondToAuthenticationResponse
from bungio.models import DestinyFireteamFinderRespondToOfferResponse
from bungio.models import BungieMembershipType
from bungio.models import DestinyFireteamFinderGetApplicationResponse
from bungio.models import DestinyFireteamFinderGetLobbyOffersResponse
from bungio.models import DestinyFireteamFinderKickPlayerRequest
from bungio.models import DestinyFireteamFinderRespondToOfferRequest
from bungio.models import DestinyFireteamFinderUpdateLobbySettingsResponse
from bungio.models import DestinyFireteamFinderApplyToListingResponse
from bungio.models import DestinyFireteamFinderSearchListingsByClanRequest
from bungio.models import DestinyFireteamFinderGetPlayerLobbiesResponse
from bungio.models import DestinyFireteamFinderSearchListingsByFiltersRequest
from bungio.models import DestinyFireteamFinderSearchListingsByFiltersResponse
from bungio.models import DestinyFireteamFinderHostLobbyRequest
from bungio.models import DestinyFireteamFinderJoinLobbyRequest


@custom_define()
class FireteamFinderRouteInterface(ClientMixin):
    async def activate_lobby(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        lobby_id: int,
        force_activation: Optional[bool] = None,
        auth: Optional[AuthData] = None,
    ) -> bool:
        """
        Activates a lobby and initializes it as an active Fireteam.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The ID of the lobby to activate.
            force_activation: Optional boolean to forcibly activate the lobby, kicking pending applicants.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.activate_lobby(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            lobby_id=lobby_id,
            force_activation=force_activation if force_activation is not None else None,
            auth=auth,
        )
        return response["Response"]

    async def activate_lobby_for_new_listing_id(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        lobby_id: int,
        force_activation: Optional[bool] = None,
        auth: Optional[AuthData] = None,
    ) -> bool:
        """
        Activates a lobby and initializes it as an active Fireteam, returning the updated Listing ID.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The ID of the lobby to activate.
            force_activation: Optional boolean to forcibly activate the lobby, kicking pending applicants.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.activate_lobby_for_new_listing_id(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            lobby_id=lobby_id,
            force_activation=force_activation if force_activation is not None else None,
            auth=auth,
        )
        return response["Response"]

    async def apply_to_listing(
        self,
        application_type: Union[DestinyFireteamFinderApplicationType, int],
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        listing_id: int,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderApplyToListingResponse:
        """
        Applies to have a character join a fireteam.

        Args:
            application_type: The type of application to apply
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            listing_id: The id of the listing to apply to
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.apply_to_listing(
            application_type=getattr(application_type, "value", application_type),
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            listing_id=listing_id,
            auth=auth,
        )
        return await DestinyFireteamFinderApplyToListingResponse.from_dict(
            data=response,
            client=self._client,
            application_type=application_type,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            listing_id=listing_id,
            auth=auth,
        )

    async def bulk_get_listing_status(
        self,
        data: dict,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderBulkGetListingStatusResponse:
        """
        Retrieves Fireteam listing statuses in bulk.

        Args:
            data: The required data for this request.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.bulk_get_listing_status(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            auth=auth,
            **data.to_dict(_return_to_bungie_case=False),
        )
        return await DestinyFireteamFinderBulkGetListingStatusResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            auth=auth,
        )

    async def get_application(
        self,
        application_id: int,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderGetApplicationResponse:
        """
        Retrieves a Fireteam application.

        Args:
            application_id:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_application(
            application_id=application_id,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            auth=auth,
        )
        return await DestinyFireteamFinderGetApplicationResponse.from_dict(
            data=response,
            client=self._client,
            application_id=application_id,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            auth=auth,
        )

    async def get_listing(self, listing_id: int, auth: Optional[AuthData] = None) -> DestinyFireteamFinderListing:
        """
        Retrieves a Fireteam listing.

        Args:
            listing_id: The ID of the listing to retrieve.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_listing(listing_id=listing_id, auth=auth)
        return await DestinyFireteamFinderListing.from_dict(
            data=response, client=self._client, listing_id=listing_id, auth=auth
        )

    async def get_listing_applications(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        listing_id: int,
        flags: Optional[int] = None,
        next_page_token: Optional[str] = None,
        page_size: Optional[int] = None,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderGetListingApplicationsResponse:
        """
        Retrieves all applications to a Fireteam Finder listing.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            listing_id: The ID of the listing whose applications to retrieve.
            flags: Optional flag representing a filter on the state of the application.
            next_page_token: An optional token from a previous response to fetch the next page of results.
            page_size: The maximum number of results to be returned with this page.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_listing_applications(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            listing_id=listing_id,
            flags=flags if flags is not None else None,
            next_page_token=next_page_token if next_page_token is not None else None,
            page_size=page_size if page_size is not None else None,
            auth=auth,
        )
        return await DestinyFireteamFinderGetListingApplicationsResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            listing_id=listing_id,
            flags=flags,
            next_page_token=next_page_token,
            page_size=page_size,
            auth=auth,
        )

    async def get_lobby(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        lobby_id: int,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderLobbyResponse:
        """
        Retrieves the information for a Fireteam lobby.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The ID of the lobby to retrieve.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_lobby(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            lobby_id=lobby_id,
            auth=auth,
        )
        return await DestinyFireteamFinderLobbyResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            lobby_id=lobby_id,
            auth=auth,
        )

    async def get_player_lobbies(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        next_page_token: Optional[str] = None,
        page_size: Optional[int] = None,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderGetPlayerLobbiesResponse:
        """
        Retrieves the information for a Fireteam lobby.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            next_page_token: An optional token from a previous response to fetch the next page of results.
            page_size: The maximum number of results to be returned with this page.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_player_lobbies(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            next_page_token=next_page_token if next_page_token is not None else None,
            page_size=page_size if page_size is not None else None,
            auth=auth,
        )
        return await DestinyFireteamFinderGetPlayerLobbiesResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            next_page_token=next_page_token,
            page_size=page_size,
            auth=auth,
        )

    async def get_player_applications(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        next_page_token: Optional[str] = None,
        page_size: Optional[int] = None,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderGetPlayerApplicationsResponse:
        """
        Retrieves Fireteam applications that this player has sent or recieved.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            next_page_token: An optional token from a previous response to fetch the next page of results.
            page_size: The maximum number of results to be returned with this page.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_player_applications(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            next_page_token=next_page_token if next_page_token is not None else None,
            page_size=page_size if page_size is not None else None,
            auth=auth,
        )
        return await DestinyFireteamFinderGetPlayerApplicationsResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            next_page_token=next_page_token,
            page_size=page_size,
            auth=auth,
        )

    async def get_player_offers(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        next_page_token: Optional[str] = None,
        page_size: Optional[int] = None,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderGetPlayerOffersResponse:
        """
        Retrieves Fireteam offers that this player has recieved.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            next_page_token: An optional token from a previous response to fetch the next page of results.
            page_size: The maximum number of results to be returned with this page.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_player_offers(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            next_page_token=next_page_token if next_page_token is not None else None,
            page_size=page_size if page_size is not None else None,
            auth=auth,
        )
        return await DestinyFireteamFinderGetPlayerOffersResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            next_page_token=next_page_token,
            page_size=page_size,
            auth=auth,
        )

    async def get_character_activity_access(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderGetCharacterActivityAccessResponse:
        """
        Retrieves the information for a Fireteam lobby.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_character_activity_access(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            auth=auth,
        )
        return await DestinyFireteamFinderGetCharacterActivityAccessResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            auth=auth,
        )

    async def get_offer(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        offer_id: int,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderOffer:
        """
        Retrieves an offer to a Fireteam lobby.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            offer_id: The unique ID of the offer.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_offer(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            offer_id=offer_id,
            auth=auth,
        )
        return await DestinyFireteamFinderOffer.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            offer_id=offer_id,
            auth=auth,
        )

    async def get_lobby_offers(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        lobby_id: int,
        next_page_token: Optional[str] = None,
        page_size: Optional[int] = None,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderGetLobbyOffersResponse:
        """
        Retrieves all offers relevant to a Fireteam lobby.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The unique ID of the lobby.
            next_page_token: An optional token from a previous response to fetch the next page of results.
            page_size: The maximum number of results to be returned with this page.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.get_lobby_offers(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            lobby_id=lobby_id,
            next_page_token=next_page_token if next_page_token is not None else None,
            page_size=page_size if page_size is not None else None,
            auth=auth,
        )
        return await DestinyFireteamFinderGetLobbyOffersResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            lobby_id=lobby_id,
            next_page_token=next_page_token,
            page_size=page_size,
            auth=auth,
        )

    async def host_lobby(
        self,
        data: DestinyFireteamFinderHostLobbyRequest,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderHostLobbyResponse:
        """
        Creates a new Fireteam lobby and Fireteam Finder listing.

        Args:
            data: The required data for this request.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.host_lobby(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            auth=auth,
            **data.to_dict(_return_to_bungie_case=False),
        )
        return await DestinyFireteamFinderHostLobbyResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            auth=auth,
        )

    async def join_lobby(
        self,
        data: DestinyFireteamFinderJoinLobbyRequest,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderLobbyResponse:
        """
        Sends a request to join an available Fireteam lobby.

        Args:
            data: The required data for this request.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.join_lobby(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            auth=auth,
            **data.to_dict(_return_to_bungie_case=False),
        )
        return await DestinyFireteamFinderLobbyResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            auth=auth,
        )

    async def kick_player(
        self,
        data: DestinyFireteamFinderKickPlayerRequest,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        lobby_id: int,
        target_membership_id: int,
        auth: Optional[AuthData] = None,
    ) -> bool:
        """
        Kicks a player from a Fireteam Finder lobby.

        Args:
            data: The required data for this request.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The ID of the lobby to kick the player from.
            target_membership_id: A valid Destiny membership ID of the player to kick.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.kick_player(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            lobby_id=lobby_id,
            target_membership_id=target_membership_id,
            auth=auth,
            **data.to_dict(_return_to_bungie_case=False),
        )
        return response["Response"]

    async def leave_application(
        self,
        application_id: int,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        auth: Optional[AuthData] = None,
    ) -> bool:
        """
        Sends a request to leave a Fireteam listing application.

        Args:
            application_id: The ID of the application to leave.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.leave_application(
            application_id=application_id,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            auth=auth,
        )
        return response["Response"]

    async def leave_lobby(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        lobby_id: int,
        auth: Optional[AuthData] = None,
    ) -> bool:
        """
        Sends a request to leave a Fireteam lobby.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The ID of the lobby to leave.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.leave_lobby(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            lobby_id=lobby_id,
            auth=auth,
        )
        return response["Response"]

    async def respond_to_application(
        self,
        data: DestinyFireteamFinderRespondToApplicationRequest,
        application_id: int,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderRespondToApplicationResponse:
        """
        Responds to an application sent to a Fireteam lobby.

        Args:
            data: The required data for this request.
            application_id: The application ID to respond to.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.respond_to_application(
            application_id=application_id,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            auth=auth,
            **data.to_dict(_return_to_bungie_case=False),
        )
        return await DestinyFireteamFinderRespondToApplicationResponse.from_dict(
            data=response,
            client=self._client,
            application_id=application_id,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            auth=auth,
        )

    async def respond_to_authentication(
        self,
        data: DestinyFireteamFinderRespondToAuthenticationRequest,
        application_id: int,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderRespondToAuthenticationResponse:
        """
        Responds to an authentication request for a Fireteam.

        Args:
            data: The required data for this request.
            application_id: The ID of the application whose authentication to confirm.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.respond_to_authentication(
            application_id=application_id,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            auth=auth,
            **data.to_dict(_return_to_bungie_case=False),
        )
        return await DestinyFireteamFinderRespondToAuthenticationResponse.from_dict(
            data=response,
            client=self._client,
            application_id=application_id,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            auth=auth,
        )

    async def respond_to_offer(
        self,
        data: DestinyFireteamFinderRespondToOfferRequest,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        offer_id: int,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderRespondToOfferResponse:
        """
        Responds to a Fireteam lobby offer.

        Args:
            data: The required data for this request.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            offer_id: The unique ID of the offer.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.respond_to_offer(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            offer_id=offer_id,
            auth=auth,
            **data.to_dict(_return_to_bungie_case=False),
        )
        return await DestinyFireteamFinderRespondToOfferResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            offer_id=offer_id,
            auth=auth,
        )

    async def search_listings_by_clan(
        self,
        data: DestinyFireteamFinderSearchListingsByClanRequest,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderSearchListingsByClanResponse:
        """
        Returns search results for available Fireteams provided a clan.

        Args:
            data: The required data for this request.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.search_listings_by_clan(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            auth=auth,
            **data.to_dict(_return_to_bungie_case=False),
        )
        return await DestinyFireteamFinderSearchListingsByClanResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            auth=auth,
        )

    async def search_listings_by_filters(
        self,
        data: DestinyFireteamFinderSearchListingsByFiltersRequest,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderSearchListingsByFiltersResponse:
        """
        Returns search results for available Fireteams provided search filters.

        Args:
            data: The required data for this request.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.search_listings_by_filters(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            auth=auth,
            **data.to_dict(_return_to_bungie_case=False),
        )
        return await DestinyFireteamFinderSearchListingsByFiltersResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            auth=auth,
        )

    async def update_lobby_settings(
        self,
        data: DestinyFireteamFinderUpdateLobbySettingsRequest,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: Union[BungieMembershipType, int],
        lobby_id: int,
        auth: Optional[AuthData] = None,
    ) -> DestinyFireteamFinderUpdateLobbySettingsResponse:
        """
        Updates the settings for a Fireteam lobby.

        Args:
            data: The required data for this request.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The ID of the lobby to update.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        response = await self._client.http.update_lobby_settings(
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=getattr(destiny_membership_type, "value", destiny_membership_type),
            lobby_id=lobby_id,
            auth=auth,
            **data.to_dict(_return_to_bungie_case=False),
        )
        return await DestinyFireteamFinderUpdateLobbySettingsResponse.from_dict(
            data=response,
            client=self._client,
            destiny_character_id=destiny_character_id,
            destiny_membership_id=destiny_membership_id,
            destiny_membership_type=destiny_membership_type,
            lobby_id=lobby_id,
            auth=auth,
        )



from typing import TYPE_CHECKING, Optional, Union

from bungio.models.base import ClientMixin, FuzzyAttrFinder, custom_define

if TYPE_CHECKING:
    # AUTOMATIC IMPORTS START
    from bungio.models import GroupOptionalConversation
    from bungio.models import BungieMembershipType
    from bungio.models import SearchResultOfGroupMemberApplication
    from bungio.models import SearchResultOfFireteamSummary
    from bungio.models import GroupResponse
    from bungio.models import DestinyMilestone
    from bungio.models import FireteamResponse
    from bungio.models import GroupEditAction
    from bungio.models import SearchResultOfGroupBan
    from bungio.models import EntityActionResult
    from bungio.models import GroupOptionalConversationAddRequest
    from bungio.models import GroupOptionalConversationEditRequest
    from bungio.models import ClanBanner
    from bungio.models import DestinyLeaderboard
    from bungio.models import FireteamPublicSearchOption
    from bungio.models import GroupApplicationRequest
    from bungio.models import DestinyClanAggregateStat
    from bungio.models import GroupOptionsEditAction
    from bungio.models import SearchResultOfGroupEditHistory
    from bungio.models import FireteamSlotSearch
    from bungio.models import GroupApplicationListRequest
    from bungio.models import FireteamPlatform
    from bungio.models import RuntimeGroupMemberType
    from bungio.models import FireteamDateRange
    from bungio.models import SearchResultOfFireteamResponse
    from bungio.models import SearchResultOfGroupMember

    # AUTOMATIC IMPORTS END
    from bungio.models.auth import AuthData

__all__ = ("DestinyClanMixin",)


@custom_define()
class DestinyClanMixin(ClientMixin, FuzzyAttrFinder):
    # DO NOT CHANGE ANY CODE BELOW. Automatically generated and overwritten

    async def get_group(self, auth: Optional["AuthData"] = None) -> "GroupResponse":
        """
        Get information about a specific group of the given ID.

        Args:
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_group(group_id=self._fuzzy_getattr("group_id"), auth=auth)

    async def get_group_optional_conversations(
        self, auth: Optional["AuthData"] = None
    ) -> list["GroupOptionalConversation"]:
        """
        Gets a list of available optional conversation channels and their settings.

        Args:
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_group_optional_conversations(
            group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def edit_group(self, data: "GroupEditAction", auth: "AuthData") -> int:
        """
        Edit an existing group. You must have suitable permissions in the group to perform this operation. This latest revision will only edit the fields you pass in - pass null for properties you want to leave unaltered.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            data: The required data for this request.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.edit_group(data=data, group_id=self._fuzzy_getattr("group_id"), auth=auth)

    async def edit_clan_banner(self, data: "ClanBanner", auth: "AuthData") -> int:
        """
        Edit an existing group's clan banner. You must have suitable permissions in the group to perform this operation. All fields are required.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            data: The required data for this request.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.edit_clan_banner(data=data, group_id=self._fuzzy_getattr("group_id"), auth=auth)

    async def edit_founder_options(self, data: "GroupOptionsEditAction", auth: "AuthData") -> int:
        """
        Edit group options only available to a founder. You must have suitable permissions in the group to perform this operation.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            data: The required data for this request.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.edit_founder_options(
            data=data, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def add_optional_conversation(self, data: "GroupOptionalConversationAddRequest", auth: "AuthData") -> int:
        """
        Add a new optional conversation/chat channel. Requires admin permissions to the group.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            data: The required data for this request.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.add_optional_conversation(
            data=data, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def edit_optional_conversation(
        self, data: "GroupOptionalConversationEditRequest", conversation_id: int, auth: "AuthData"
    ) -> int:
        """
        Edit the settings of an optional conversation/chat channel. Requires admin permissions to the group.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            data: The required data for this request.
            conversation_id: Conversation Id of the channel being edited.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.edit_optional_conversation(
            data=data, conversation_id=conversation_id, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def get_members_of_group(
        self,
        currentpage: int,
        member_type: Union["RuntimeGroupMemberType", int],
        name_search: str,
        auth: Optional["AuthData"] = None,
    ) -> "SearchResultOfGroupMember":
        """
        Get the list of members in a given group.

        Args:
            currentpage: Page number (starting with 1). Each page has a fixed size of 50 items per page.
            member_type: Filter out other member types. Use None for all members.
            name_search: The name fragment upon which a search should be executed for members with matching display or unique names.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_members_of_group(
            currentpage=currentpage,
            group_id=self._fuzzy_getattr("group_id"),
            member_type=member_type,
            name_search=name_search,
            auth=auth,
        )

    async def get_admins_and_founder_of_group(
        self, currentpage: int, auth: Optional["AuthData"] = None
    ) -> "SearchResultOfGroupMember":
        """
        Get the list of members in a given group who are of admin level or higher.

        Args:
            currentpage: Page number (starting with 1). Each page has a fixed size of 50 items per page.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_admins_and_founder_of_group(
            currentpage=currentpage, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def get_banned_members_of_group(self, currentpage: int, auth: "AuthData") -> "SearchResultOfGroupBan":
        """
        Get the list of banned members in a given group. Only accessible to group Admins and above. Not applicable to all groups. Check group features.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            currentpage: Page number (starting with 1). Each page has a fixed size of 50 entries.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_banned_members_of_group(
            currentpage=currentpage, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def get_group_edit_history(self, currentpage: int, auth: "AuthData") -> "SearchResultOfGroupEditHistory":
        """
        Get the list of edits made to a given group. Only accessible to group Admins and above.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            currentpage: Page number (starting with 1). Each page has a fixed size of 50 entries.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_group_edit_history(
            currentpage=currentpage, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def abdicate_foundership(
        self,
        founder_id_new: int,
        membership_type: Union["BungieMembershipType", int],
        auth: Optional["AuthData"] = None,
    ) -> bool:
        """
        An administrative method to allow the founder of a group or clan to give up their position to another admin permanently.

        Args:
            founder_id_new: The new founder for this group. Must already be a group admin.
            membership_type: Membership type of the provided founderIdNew.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.abdicate_foundership(
            founder_id_new=founder_id_new,
            group_id=self._fuzzy_getattr("group_id"),
            membership_type=membership_type,
            auth=auth,
        )

    async def get_pending_memberships(
        self, currentpage: int, auth: "AuthData"
    ) -> "SearchResultOfGroupMemberApplication":
        """
        Get the list of users who are awaiting a decision on their application to join a given group. Modified to include application info.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            currentpage: Page number (starting with 1). Each page has a fixed size of 50 items per page.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_pending_memberships(
            currentpage=currentpage, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def get_invited_individuals(
        self, currentpage: int, auth: "AuthData"
    ) -> "SearchResultOfGroupMemberApplication":
        """
        Get the list of users who have been invited into the group.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            currentpage: Page number (starting with 1). Each page has a fixed size of 50 items per page.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_invited_individuals(
            currentpage=currentpage, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def approve_all_pending(
        self, data: "GroupApplicationRequest", auth: "AuthData"
    ) -> list["EntityActionResult"]:
        """
        Approve all of the pending users for the given group.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            data: The required data for this request.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.approve_all_pending(
            data=data, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def deny_all_pending(self, data: "GroupApplicationRequest", auth: "AuthData") -> list["EntityActionResult"]:
        """
        Deny all of the pending users for the given group.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            data: The required data for this request.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.deny_all_pending(data=data, group_id=self._fuzzy_getattr("group_id"), auth=auth)

    async def approve_pending_for_list(
        self, data: "GroupApplicationListRequest", auth: "AuthData"
    ) -> list["EntityActionResult"]:
        """
        Approve all of the pending users for the given group.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            data: The required data for this request.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.approve_pending_for_list(
            data=data, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def deny_pending_for_list(
        self, data: "GroupApplicationListRequest", auth: "AuthData"
    ) -> list["EntityActionResult"]:
        """
        Deny all of the pending users for the given group that match the passed-in .

        Warning: Requires Authentication.
            Required oauth2 scopes: AdminGroups

        Args:
            data: The required data for this request.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.deny_pending_for_list(
            data=data, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def get_clan_weekly_reward_state(self, auth: Optional["AuthData"] = None) -> "DestinyMilestone":
        """
        Returns information on the weekly clan rewards and if the clan has earned them or not. Note that this will always report rewards as not redeemed.

        Args:
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_clan_weekly_reward_state(group_id=self._fuzzy_getattr("group_id"), auth=auth)

    async def get_clan_leaderboards(
        self, maxtop: int, modes: str, statid: str, auth: Optional["AuthData"] = None
    ) -> dict[str, dict[str, "DestinyLeaderboard"]]:
        """
        Gets leaderboards with the signed in user's friends and the supplied destinyMembershipId as the focus. PREVIEW: This endpoint is still in beta, and may experience rough edges. The schema is in final form, but there may be bugs that prevent desirable operation.

        Args:
            maxtop: Maximum number of top players to return. Use a large number to get entire leaderboard.
            modes: List of game modes for which to get leaderboards. See the documentation for DestinyActivityModeType for valid values, and pass in string representation, comma delimited.
            statid: ID of stat to return rather than returning all Leaderboard stats.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_clan_leaderboards(
            group_id=self._fuzzy_getattr("group_id"), maxtop=maxtop, modes=modes, statid=statid, auth=auth
        )

    async def get_clan_aggregate_stats(
        self, modes: str, auth: Optional["AuthData"] = None
    ) -> list["DestinyClanAggregateStat"]:
        """
        Gets aggregated stats for a clan using the same categories as the clan leaderboards. PREVIEW: This endpoint is still in beta, and may experience rough edges. The schema is in final form, but there may be bugs that prevent desirable operation.

        Args:
            modes: List of game modes for which to get leaderboards. See the documentation for DestinyActivityModeType for valid values, and pass in string representation, comma delimited.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_clan_aggregate_stats(
            group_id=self._fuzzy_getattr("group_id"), modes=modes, auth=auth
        )

    async def get_active_private_clan_fireteam_count(self, auth: "AuthData") -> int:
        """
        Gets a count of all active non-public fireteams for the specified clan. Maximum value returned is 25.

        Warning: Requires Authentication.
            Required oauth2 scopes: ReadGroups

        Args:
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_active_private_clan_fireteam_count(
            group_id=self._fuzzy_getattr("group_id"), auth=auth
        )

    async def get_available_clan_fireteams(
        self,
        activity_type: int,
        date_range: Union["FireteamDateRange", int],
        page: int,
        platform: Union["FireteamPlatform", int],
        public_only: Union["FireteamPublicSearchOption", int],
        slot_filter: Union["FireteamSlotSearch", int],
        auth: "AuthData",
        exclude_immediate: bool,
        lang_filter: str,
    ) -> "SearchResultOfFireteamSummary":
        """
        Gets a listing of all of this clan's fireteams that are have available slots. Caller is not checked for join criteria so caching is maximized.

        Warning: Requires Authentication.
            Required oauth2 scopes: ReadGroups

        Args:
            activity_type: The activity type to filter by.
            date_range: The date range to grab available fireteams.
            page: Zero based page
            platform: The platform filter.
            public_only: Determines public/private filtering.
            slot_filter: Filters based on available slots
            auth: Authentication information.
            exclude_immediate: If you wish the result to exclude immediate fireteams, set this to true. Immediate-only can be forced using the dateRange enum.
            lang_filter: An optional language filter.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_available_clan_fireteams(
            activity_type=activity_type,
            date_range=date_range,
            group_id=self._fuzzy_getattr("group_id"),
            page=page,
            platform=platform,
            public_only=public_only,
            slot_filter=slot_filter,
            auth=auth,
            exclude_immediate=exclude_immediate,
            lang_filter=lang_filter,
        )

    async def get_my_clan_fireteams(
        self,
        include_closed: bool,
        page: int,
        platform: Union["FireteamPlatform", int],
        auth: "AuthData",
        group_filter: bool,
        lang_filter: str,
    ) -> "SearchResultOfFireteamResponse":
        """
        Gets a listing of all fireteams that caller is an applicant, a member, or an alternate of.

        Warning: Requires Authentication.
            Required oauth2 scopes: ReadGroups

        Args:
            include_closed: If true, return fireteams that have been closed.
            page: Deprecated parameter, ignored.
            platform: The platform filter.
            auth: Authentication information.
            group_filter: If true, filter by clan. Otherwise, ignore the clan and show all of the user's fireteams.
            lang_filter: An optional language filter.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_my_clan_fireteams(
            group_id=self._fuzzy_getattr("group_id"),
            include_closed=include_closed,
            page=page,
            platform=platform,
            auth=auth,
            group_filter=group_filter,
            lang_filter=lang_filter,
        )

    async def get_clan_fireteam(self, fireteam_id: int, auth: "AuthData") -> "FireteamResponse":
        """
        Gets a specific fireteam.

        Warning: Requires Authentication.
            Required oauth2 scopes: ReadGroups

        Args:
            fireteam_id: The unique id of the fireteam.
            auth: Authentication information.

        Returns:
            The model which is returned by bungie. [General endpoint information.](https://bungie-net.github.io/multi/index.html)
        """

        return await self._client.api.get_clan_fireteam(
            fireteam_id=fireteam_id, group_id=self._fuzzy_getattr("group_id"), auth=auth
        )



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from datetime import datetime
from typing import Optional, Any, Union, TYPE_CHECKING

from bungio.utils import enum_converter
from bungio.models.base import BaseModel, BaseEnum, BaseFlagEnum, HashObject, ManifestModel, custom_define, custom_field


if TYPE_CHECKING:
    from bungio.models import BungieMembershipType
    from bungio.models import DestinyActivityModeType
    from bungio.models import DestinyInventoryItemDefinition
    from bungio.models import DestinyRaceDefinition
    from bungio.models import DestinyHistoricalStatsPeriodGroup
    from bungio.models import DestinyClassDefinition
    from bungio.models import DestinyActivityDefinition
    from bungio.models import UserInfoCard
    from bungio.models import DestinyGenderDefinition


@custom_define()
class DestinyPostGameCarnageReportData(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        activity_details: Details about the activity.
        activity_was_started_from_beginning: True if the activity was started from the beginning, if that information is available and the activity was played post Witch Queen release.
        entries: Collection of players and their data for this activity.
        period: Date and time for the activity.
        starting_phase_index: If this activity has "phases", this is the phase at which the activity was started. This value is only valid for activities before the Beyond Light expansion shipped. Subsequent activities will not have a valid value here.
        teams: Collection of stats for the player in this activity.
    """

    activity_details: "DestinyHistoricalStatsActivity" = custom_field()
    activity_was_started_from_beginning: bool = custom_field()
    entries: list["DestinyPostGameCarnageReportEntry"] = custom_field(
        metadata={"type": """list[DestinyPostGameCarnageReportEntry]"""}
    )
    period: datetime = custom_field()
    starting_phase_index: int = custom_field()
    teams: list["DestinyPostGameCarnageReportTeamEntry"] = custom_field(
        metadata={"type": """list[DestinyPostGameCarnageReportTeamEntry]"""}
    )


@custom_define()
class DestinyHistoricalStatsActivity(BaseModel):
    """
    Summary information about the activity that was played.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        director_activity_hash: The unique hash identifier of the DestinyActivityDefinition that was played.
        instance_id: The unique identifier for this *specific* match that was played. This value can be used to get additional data about this activity such as who else was playing via the GetPostGameCarnageReport endpoint.
        is_private: Whether or not the match was a private match.
        membership_type: The Membership Type indicating the platform on which this match was played.
        mode: Indicates the most specific game mode of the activity that we could find.
        modes: The list of all Activity Modes to which this activity applies, including aggregates. This will let you see, for example, whether the activity was both Clash and part of the Trials of the Nine event.
        reference_id: The unique hash identifier of the DestinyActivityDefinition that was played. If I had this to do over, it'd be named activityHash. Too late now.
        manifest_director_activity_hash: Manifest information for `director_activity_hash`
        manifest_reference_id: Manifest information for `reference_id`
    """

    director_activity_hash: int = custom_field()
    instance_id: int = custom_field(metadata={"int64": True})
    is_private: bool = custom_field()
    membership_type: Union["BungieMembershipType", int] = custom_field(converter=enum_converter("BungieMembershipType"))
    mode: Union["DestinyActivityModeType", int] = custom_field(converter=enum_converter("DestinyActivityModeType"))
    modes: list[Union["DestinyActivityModeType", int]] = custom_field(
        converter=enum_converter("DestinyActivityModeType")
    )
    reference_id: int = custom_field()
    manifest_director_activity_hash: Optional["DestinyActivityDefinition"] = custom_field(default=None)
    manifest_reference_id: Optional["DestinyActivityDefinition"] = custom_field(default=None)


@custom_define()
class DestinyPostGameCarnageReportEntry(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        character_id: ID of the player's character used in the activity.
        extended: Extended data extracted from the activity blob.
        player: Identity details of the player
        score: Score of the player if available
        standing: Standing of the player
        values: Collection of stats for the player in this activity.
    """

    character_id: int = custom_field(metadata={"int64": True})
    extended: "DestinyPostGameCarnageReportExtendedData" = custom_field()
    player: "DestinyPlayer" = custom_field()
    score: "DestinyHistoricalStatsValue" = custom_field()
    standing: int = custom_field()
    values: dict[str, "DestinyHistoricalStatsValue"] = custom_field(
        metadata={"type": """dict[str, DestinyHistoricalStatsValue]"""}
    )


@custom_define()
class DestinyHistoricalStatsValue(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        activity_id: When a stat represents the best, most, longest, fastest or some other personal best, the actual activity ID where that personal best was established is available on this property.
        basic: Basic stat value.
        pga: Per game average for the statistic, if applicable
        stat_id: Unique ID for this stat
        weighted: Weighted value of the stat if a weight greater than 1 has been assigned.
    """

    activity_id: int = custom_field(metadata={"int64": True})
    basic: "DestinyHistoricalStatsValuePair" = custom_field()
    pga: "DestinyHistoricalStatsValuePair" = custom_field()
    stat_id: str = custom_field()
    weighted: "DestinyHistoricalStatsValuePair" = custom_field()


@custom_define()
class DestinyHistoricalStatsValuePair(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        display_value: Localized formated version of the value.
        value: Raw value of the statistic
    """

    display_value: str = custom_field()
    value: float = custom_field()


@custom_define()
class DestinyPlayer(BaseModel):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        bungie_net_user_info: Details about the player as they are known on BungieNet. This will be undefined if the player has marked their credential private, or does not have a BungieNet account.
        character_class: Class of the character if applicable and available.
        character_level: Level of the character if available. Zero if it is not available.
        clan_name: Current clan name for the player. This value may be null or an empty string if the user does not have a clan.
        clan_tag: Current clan tag for the player. This value may be null or an empty string if the user does not have a clan.
        class_hash: _No description given by bungie._
        destiny_user_info: Details about the player as they are known in game (platform display name, Destiny emblem)
        emblem_hash: If we know the emblem's hash, this can be used to look up the player's emblem at the time of a match when receiving PGCR data, or otherwise their currently equipped emblem (if we are able to obtain it).
        gender_hash: _No description given by bungie._
        light_level: Light Level of the character if available. Zero if it is not available.
        race_hash: _No description given by bungie._
        manifest_class_hash: Manifest information for `class_hash`
        manifest_emblem_hash: Manifest information for `emblem_hash`
        manifest_gender_hash: Manifest information for `gender_hash`
        manifest_race_hash: Manifest information for `race_hash`
    """

    bungie_net_user_info: "UserInfoCard" = custom_field()
    character_class: str = custom_field()
    character_level: int = custom_field()
    clan_name: str = custom_field()
    clan_tag: str = custom_field()
    class_hash: int = custom_field()
    destiny_user_info: "UserInfoCard" = custom_field()
    emblem_hash: int = custom_field()
    gender_hash: int = custom_field()
    light_level: int = custom_field()
    race_hash: int = custom_field()
    manifest_class_hash: Optional["DestinyClassDefinition"] = custom_field(default=None)
    manifest_emblem_hash: Optional["DestinyInventoryItemDefinition"] = custom_field(default=None)
    manifest_gender_hash: Optional["DestinyGenderDefinition"] = custom_field(default=None)
    manifest_race_hash: Optional["DestinyRaceDefinition"] = custom_field(default=None)


@custom_define()
class DestinyPostGameCarnageReportExtendedData(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        values: Collection of stats for the player in this activity.
        weapons: List of weapons and their perspective values.
    """

    values: dict[str, "DestinyHistoricalStatsValue"] = custom_field(
        metadata={"type": """dict[str, DestinyHistoricalStatsValue]"""}
    )
    weapons: list["DestinyHistoricalWeaponStats"] = custom_field(
        metadata={"type": """list[DestinyHistoricalWeaponStats]"""}
    )


@custom_define()
class DestinyHistoricalWeaponStats(BaseModel):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        reference_id: The hash ID of the item definition that describes the weapon.
        values: Collection of stats for the period.
        manifest_reference_id: Manifest information for `reference_id`
    """

    reference_id: int = custom_field()
    values: dict[str, "DestinyHistoricalStatsValue"] = custom_field(
        metadata={"type": """dict[str, DestinyHistoricalStatsValue]"""}
    )
    manifest_reference_id: Optional["DestinyInventoryItemDefinition"] = custom_field(default=None)


@custom_define()
class DestinyPostGameCarnageReportTeamEntry(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        score: Score earned by the team
        standing: Team's standing relative to other teams.
        team_id: Integer ID for the team.
        team_name: Alpha or Bravo
    """

    score: "DestinyHistoricalStatsValue" = custom_field()
    standing: "DestinyHistoricalStatsValue" = custom_field()
    team_id: int = custom_field()
    team_name: str = custom_field()


@custom_define()
class DestinyLeaderboard(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        entries: _No description given by bungie._
        stat_id: _No description given by bungie._
    """

    entries: list["DestinyLeaderboardEntry"] = custom_field(metadata={"type": """list[DestinyLeaderboardEntry]"""})
    stat_id: str = custom_field()


@custom_define()
class DestinyLeaderboardEntry(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        character_id: ID of the player's best character for the reported stat.
        player: Identity details of the player
        rank: Where this player ranks on the leaderboard. A value of 1 is the top rank.
        value: Value of the stat for this player
    """

    character_id: int = custom_field(metadata={"int64": True})
    player: "DestinyPlayer" = custom_field()
    rank: int = custom_field()
    value: "DestinyHistoricalStatsValue" = custom_field()


@custom_define()
class DestinyLeaderboardResults(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        focus_character_id: Indicate the character ID of the character that is the focal point of the provided leaderboards. May be null, in which case any character from the focus membership can appear in the provided leaderboards.
        focus_membership_id: Indicate the membership ID of the account that is the focal point of the provided leaderboards.
    """

    focus_character_id: int = custom_field(metadata={"int64": True})
    focus_membership_id: int = custom_field(metadata={"int64": True})


@custom_define()
class DestinyClanAggregateStat(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        mode: The id of the mode of stats (allPvp, allPvE, etc)
        stat_id: The id of the stat
        value: Value of the stat for this player
    """

    mode: Union["DestinyActivityModeType", int] = custom_field(converter=enum_converter("DestinyActivityModeType"))
    stat_id: str = custom_field()
    value: "DestinyHistoricalStatsValue" = custom_field()


@custom_define()
class DestinyHistoricalStatsByPeriod(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        all_time: _No description given by bungie._
        all_time_tier1: _No description given by bungie._
        all_time_tier2: _No description given by bungie._
        all_time_tier3: _No description given by bungie._
        daily: _No description given by bungie._
        monthly: _No description given by bungie._
    """

    all_time: dict[str, "DestinyHistoricalStatsValue"] = custom_field(
        metadata={"type": """dict[str, DestinyHistoricalStatsValue]"""}
    )
    all_time_tier1: dict[str, "DestinyHistoricalStatsValue"] = custom_field(
        metadata={"type": """dict[str, DestinyHistoricalStatsValue]"""}
    )
    all_time_tier2: dict[str, "DestinyHistoricalStatsValue"] = custom_field(
        metadata={"type": """dict[str, DestinyHistoricalStatsValue]"""}
    )
    all_time_tier3: dict[str, "DestinyHistoricalStatsValue"] = custom_field(
        metadata={"type": """dict[str, DestinyHistoricalStatsValue]"""}
    )
    daily: list["DestinyHistoricalStatsPeriodGroup"] = custom_field(
        metadata={"type": """list[DestinyHistoricalStatsPeriodGroup]"""}
    )
    monthly: list["DestinyHistoricalStatsPeriodGroup"] = custom_field(
        metadata={"type": """list[DestinyHistoricalStatsPeriodGroup]"""}
    )


@custom_define()
class OverwrittenDestinyHistoricalStatsPeriodGroup(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        activity_details: If the period group is for a specific activity, this property will be set.
        period: Period for the group. If the stat periodType is day, then this will have a specific day. If the type is monthly, then this value will be the first day of the applicable month. This value is not set when the periodType is 'all time'.
        values: Collection of stats for the period.
    """

    activity_details: "DestinyHistoricalStatsActivity" = custom_field()
    period: datetime = custom_field()
    values: dict[str, "DestinyHistoricalStatsValue"] = custom_field(
        metadata={"type": """dict[str, DestinyHistoricalStatsValue]"""}
    )


@custom_define()
class DestinyHistoricalStatsAccountResult(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        characters: _No description given by bungie._
        merged_all_characters: _No description given by bungie._
        merged_deleted_characters: _No description given by bungie._
    """

    characters: list["DestinyHistoricalStatsPerCharacter"] = custom_field(
        metadata={"type": """list[DestinyHistoricalStatsPerCharacter]"""}
    )
    merged_all_characters: "DestinyHistoricalStatsWithMerged" = custom_field()
    merged_deleted_characters: "DestinyHistoricalStatsWithMerged" = custom_field()


@custom_define()
class DestinyHistoricalStatsWithMerged(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        merged: _No description given by bungie._
        results: _No description given by bungie._
    """

    merged: "DestinyHistoricalStatsByPeriod" = custom_field()
    results: dict[str, "DestinyHistoricalStatsByPeriod"] = custom_field(
        metadata={"type": """dict[str, DestinyHistoricalStatsByPeriod]"""}
    )


@custom_define()
class DestinyHistoricalStatsPerCharacter(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        character_id: _No description given by bungie._
        deleted: _No description given by bungie._
        merged: _No description given by bungie._
        results: _No description given by bungie._
    """

    character_id: int = custom_field(metadata={"int64": True})
    deleted: bool = custom_field()
    merged: "DestinyHistoricalStatsByPeriod" = custom_field()
    results: dict[str, "DestinyHistoricalStatsByPeriod"] = custom_field(
        metadata={"type": """dict[str, DestinyHistoricalStatsByPeriod]"""}
    )


@custom_define()
class DestinyActivityHistoryResults(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        activities: List of activities, the most recent activity first.
    """

    activities: list["DestinyHistoricalStatsPeriodGroup"] = custom_field(
        metadata={"type": """list[DestinyHistoricalStatsPeriodGroup]"""}
    )


@custom_define()
class DestinyHistoricalWeaponStatsData(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        weapons: List of weapons and their perspective values.
    """

    weapons: list["DestinyHistoricalWeaponStats"] = custom_field(
        metadata={"type": """list[DestinyHistoricalWeaponStats]"""}
    )


@custom_define()
class DestinyAggregateActivityResults(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        activities: List of all activities the player has participated in.
    """

    activities: list["DestinyAggregateActivityStats"] = custom_field(
        metadata={"type": """list[DestinyAggregateActivityStats]"""}
    )


@custom_define()
class DestinyAggregateActivityStats(BaseModel):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activity_hash: Hash ID that can be looked up in the DestinyActivityTable.
        values: Collection of stats for the player in this activity.
        manifest_activity_hash: Manifest information for `activity_hash`
    """

    activity_hash: int = custom_field()
    values: dict[str, "DestinyHistoricalStatsValue"] = custom_field(
        metadata={"type": """dict[str, DestinyHistoricalStatsValue]"""}
    )
    manifest_activity_hash: Optional["DestinyActivityDefinition"] = custom_field(default=None)



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from datetime import datetime
from typing import Optional, Union, TYPE_CHECKING

from bungio.utils import enum_converter
from bungio.models.base import BaseModel, custom_define, custom_field


if TYPE_CHECKING:
    from bungio.models import DestinyVendorItemState
    from bungio.models import DestinyInventoryItemDefinition
    from bungio.models import DestinyUnlockStatus
    from bungio.models import DestinyItemQuantity
    from bungio.models import VendorItemStatus
    from bungio.models import DestinyProgression
    from bungio.models import DestinyVendorDefinition


@custom_define()
class DestinyVendorComponent(BaseModel):
    """
    This component contains essential/summary information about the vendor.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        can_purchase: If True, you can purchase from the Vendor.
        enabled: If True, the Vendor is currently accessible.  If False, they may not actually be visible in the world at the moment.
        next_refresh_date: The date when this vendor's inventory will next rotate/refresh. Note that this is distinct from the date ranges that the vendor is visible/available in-game: this field indicates the specific time when the vendor's available items refresh and rotate, regardless of whether the vendor is actually available at that time. Unfortunately, these two values may be (and are, for the case of important vendors like Xur) different. Issue https://github.com/Bungie-net/api/issues/353 is tracking a fix to start providing visibility date ranges where possible in addition to this refresh date, so that all important dates for vendors are available for use.
        progression: If the Vendor has a related Reputation, this is the Progression data that represents the character's Reputation level with this Vendor.
        seasonal_rank: If this vendor has a seasonal rank, this will be the calculated value of that rank. How nice is that? I mean, that's pretty sweeet. It's a whole 32 bit integer.
        vendor_hash: The unique identifier for the vendor. Use it to look up their DestinyVendorDefinition.
        vendor_location_index: An index into the vendor definition's "locations" property array, indicating which location they are at currently. If -1, then the vendor has no known location (and you may choose not to show them in your UI as a result. I mean, it's your bag honey)
        manifest_vendor_hash: Manifest information for `vendor_hash`
    """

    can_purchase: bool = custom_field()
    enabled: bool = custom_field()
    next_refresh_date: datetime = custom_field()
    progression: "DestinyProgression" = custom_field()
    seasonal_rank: int = custom_field()
    vendor_hash: int = custom_field()
    vendor_location_index: int = custom_field()
    manifest_vendor_hash: Optional["DestinyVendorDefinition"] = custom_field(default=None)


@custom_define()
class DestinyVendorCategoriesComponent(BaseModel):
    """
    A vendor can have many categories of items that they sell. This component will return the category information for available items, as well as the index into those items in the user's sale item list. Note that, since both the category and items are indexes, this data is Content Version dependent. Be sure to check that your content is up to date before using this data. This is an unfortunate, but permanent, limitation of Vendor data.

    None
    Attributes:
        categories: The list of categories for items that the vendor sells, in rendering order. These categories each point to a "display category" in the displayCategories property of the DestinyVendorDefinition, as opposed to the other categories.
    """

    categories: list["DestinyVendorCategory"] = custom_field(metadata={"type": """list[DestinyVendorCategory]"""})


@custom_define()
class DestinyVendorCategory(BaseModel):
    """
    Information about the category and items currently sold in that category.

    None
    Attributes:
        display_category_index: An index into the DestinyVendorDefinition.displayCategories property, so you can grab the display data for this category.
        item_indexes: An ordered list of indexes into items being sold in this category (DestinyVendorDefinition.itemList) which will contain more information about the items being sold themselves. Can also be used to index into DestinyVendorSaleItemComponent data, if you asked for that data to be returned.
    """

    display_category_index: int = custom_field()
    item_indexes: list[int] = custom_field(metadata={"type": """list[int]"""})


@custom_define()
class DestinyVendorSaleItemComponent(BaseModel):
    """
    Request this component if you want the details about an item being sold in relation to the character making the request: whether the character can buy it, whether they can afford it, and other data related to purchasing the item. Note that if you want instance, stats, etc... data for the item, you'll have to request additional components such as ItemInstances, ItemPerks etc... and acquire them from the DestinyVendorResponse's "items" property.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        api_purchasable: If true, this item can be purchased through the Bungie.net API.
        augments: A flags enumeration value representing the current state of any "state modifiers" on the item being sold. These are meant to correspond with some sort of visual indicator as to the augmentation: for instance, if an item is on sale or if you already own the item in question. Determining how you want to represent these in your own app (or if you even want to) is an exercise left for the reader.
        costs: A summary of the current costs of the item.
        failure_indexes: Indexes in to the "failureStrings" lookup table in DestinyVendorDefinition for the given Vendor. Gives some more reliable failure information for why you can't purchase an item. It is preferred to use these over requiredUnlocks and unlockStatuses: the latter are provided mostly in case someone can do something interesting with it that I didn't anticipate.
        item_hash: The hash of the item being sold, as a quick shortcut for looking up the DestinyInventoryItemDefinition of the sale item.
        item_value_visibility: If available, a list that describes which item values (rewards) should be shown (true) or hidden (false).
        override_next_refresh_date: If this item has its own custom date where it may be removed from the Vendor's rotation, this is that date. Note that there's not actually any guarantee that it will go away: it could be chosen again and end up still being in the Vendor's sale items! But this is the next date where that test will occur, and is also the date that the game shows for availability on things like Bounties being sold. So it's the best we can give.
        override_style_item_hash: If populated, this is the hash of the item whose icon (and other secondary styles, but *not* the human readable strings) should override whatever icons/styles are on the item being sold. If you don't do this, certain items whose styles are being overridden by socketed items - such as the "Recycle Shader" item - would show whatever their default icon/style is, and it wouldn't be pretty or look accurate.
        quantity: How much of the item you'll be getting.
        required_unlocks: If you can't buy the item due to a complex character state, these will be hashes for DestinyUnlockDefinitions that you can check to see messages regarding the failure (if the unlocks have human readable information: it is not guaranteed that Unlocks will have human readable strings, and your application will have to handle that) Prefer using failureIndexes instead. These are provided for informational purposes, but have largely been supplanted by failureIndexes.
        sale_status: A flag indicating whether the requesting character can buy the item, and if not the reasons why the character can't buy it.
        unlock_statuses: If any complex unlock states are checked in determining purchasability, these will be returned here along with the status of the unlock check. Prefer using failureIndexes instead. These are provided for informational purposes, but have largely been supplanted by failureIndexes.
        vendor_item_index: The index into the DestinyVendorDefinition.itemList property. Note that this means Vendor data *is* Content Version dependent: make sure you have the latest content before you use Vendor data, or these indexes may mismatch.  Most systems avoid this problem, but Vendors is one area where we are unable to reasonably avoid content dependency at the moment.
        manifest_item_hash: Manifest information for `item_hash`
        manifest_override_style_item_hash: Manifest information for `override_style_item_hash`
    """

    api_purchasable: bool = custom_field()
    augments: Union["DestinyVendorItemState", int] = custom_field(converter=enum_converter("DestinyVendorItemState"))
    costs: list["DestinyItemQuantity"] = custom_field(metadata={"type": """list[DestinyItemQuantity]"""})
    failure_indexes: list[int] = custom_field(metadata={"type": """list[int]"""})
    item_hash: int = custom_field()
    item_value_visibility: list[bool] = custom_field(metadata={"type": """list[bool]"""})
    override_next_refresh_date: datetime = custom_field()
    override_style_item_hash: int = custom_field()
    quantity: int = custom_field()
    required_unlocks: list[int] = custom_field(metadata={"type": """list[int]"""})
    sale_status: Union["VendorItemStatus", int] = custom_field(converter=enum_converter("VendorItemStatus"))
    unlock_statuses: list["DestinyUnlockStatus"] = custom_field(metadata={"type": """list[DestinyUnlockStatus]"""})
    vendor_item_index: int = custom_field()
    manifest_item_hash: Optional["DestinyInventoryItemDefinition"] = custom_field(default=None)
    manifest_override_style_item_hash: Optional["DestinyInventoryItemDefinition"] = custom_field(default=None)



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import Optional, TYPE_CHECKING

from bungio.models.base import BaseModel, custom_define, custom_field


if TYPE_CHECKING:
    from bungio.models import DestinyPresentationNodeDefinition
    from bungio.models import DestinyObjectiveProgress


@custom_define()
class DestinyMetricsComponent(BaseModel):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        metrics: _No description given by bungie._
        metrics_root_node_hash: _No description given by bungie._
        manifest_metrics_root_node_hash: Manifest information for `metrics_root_node_hash`
    """

    metrics: dict[int, "DestinyMetricComponent"] = custom_field(
        metadata={"type": """dict[int, DestinyMetricComponent]"""}
    )
    metrics_root_node_hash: int = custom_field()
    manifest_metrics_root_node_hash: Optional["DestinyPresentationNodeDefinition"] = custom_field(default=None)


@custom_define()
class DestinyMetricComponent(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        invisible: _No description given by bungie._
        objective_progress: _No description given by bungie._
    """

    invisible: bool = custom_field()
    objective_progress: "DestinyObjectiveProgress" = custom_field()



from datetime import datetime
from typing import Callable, Coroutine, Optional, Any, Union

from bungio.http.route import Route
from bungio.models.auth import AuthData


class FireteamFinderRouteHttpRequests:
    request: Callable[..., Coroutine]

    async def activate_lobby(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        lobby_id: int,
        force_activation: Optional[bool] = None,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Activates a lobby and initializes it as an active Fireteam.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The ID of the lobby to activate.
            force_activation: Optional boolean to forcibly activate the lobby, kicking pending applicants.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/FireteamFinder/Lobby/Activate/{lobby_id}/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="POST",
                force_activation=force_activation,
                auth=auth,
            )
        )

    async def activate_lobby_for_new_listing_id(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        lobby_id: int,
        force_activation: Optional[bool] = None,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Activates a lobby and initializes it as an active Fireteam, returning the updated Listing ID.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The ID of the lobby to activate.
            force_activation: Optional boolean to forcibly activate the lobby, kicking pending applicants.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/FireteamFinder/Lobby/ActivateForNewListingId/{lobby_id}/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="POST",
                force_activation=force_activation,
                auth=auth,
            )
        )

    async def apply_to_listing(
        self,
        application_type: int,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        listing_id: int,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Applies to have a character join a fireteam.

        Args:
            application_type: The type of application to apply
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            listing_id: The id of the listing to apply to
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/FireteamFinder/Listing/{listing_id}/Apply/{application_type}/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="POST",
                auth=auth,
            )
        )

    async def bulk_get_listing_status(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Retrieves Fireteam listing statuses in bulk.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/FireteamFinder/Listing/BulkStatus/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="POST",
                auth=auth,
            )
        )

    async def get_application(
        self,
        application_id: int,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Retrieves a Fireteam application.

        Args:
            application_id:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/FireteamFinder/Application/{application_id}/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="GET",
                auth=auth,
            )
        )

    async def get_listing(self, listing_id: int, auth: Optional[AuthData] = None, *args, **kwargs) -> dict:
        """
        Retrieves a Fireteam listing.

        Args:
            listing_id: The ID of the listing to retrieve.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(Route(path=f"/FireteamFinder/Listing/{listing_id}/", method="GET", auth=auth))

    async def get_listing_applications(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        listing_id: int,
        flags: Optional[int] = None,
        next_page_token: Optional[str] = None,
        page_size: Optional[int] = None,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Retrieves all applications to a Fireteam Finder listing.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            listing_id: The ID of the listing whose applications to retrieve.
            flags: Optional flag representing a filter on the state of the application.
            next_page_token: An optional token from a previous response to fetch the next page of results.
            page_size: The maximum number of results to be returned with this page.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/FireteamFinder/Listing/{listing_id}/Applications/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="GET",
                flags=flags,
                next_page_token=next_page_token,
                page_size=page_size,
                auth=auth,
            )
        )

    async def get_lobby(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        lobby_id: int,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Retrieves the information for a Fireteam lobby.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The ID of the lobby to retrieve.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/FireteamFinder/Lobby/{lobby_id}/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="GET",
                auth=auth,
            )
        )

    async def get_player_lobbies(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        next_page_token: Optional[str] = None,
        page_size: Optional[int] = None,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Retrieves the information for a Fireteam lobby.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            next_page_token: An optional token from a previous response to fetch the next page of results.
            page_size: The maximum number of results to be returned with this page.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/FireteamFinder/PlayerLobbies/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="GET",
                next_page_token=next_page_token,
                page_size=page_size,
                auth=auth,
            )
        )

    async def get_player_applications(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        next_page_token: Optional[str] = None,
        page_size: Optional[int] = None,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Retrieves Fireteam applications that this player has sent or recieved.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            next_page_token: An optional token from a previous response to fetch the next page of results.
            page_size: The maximum number of results to be returned with this page.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/FireteamFinder/PlayerApplications/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="GET",
                next_page_token=next_page_token,
                page_size=page_size,
                auth=auth,
            )
        )

    async def get_player_offers(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        next_page_token: Optional[str] = None,
        page_size: Optional[int] = None,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Retrieves Fireteam offers that this player has recieved.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            next_page_token: An optional token from a previous response to fetch the next page of results.
            page_size: The maximum number of results to be returned with this page.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/FireteamFinder/PlayerOffers/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="GET",
                next_page_token=next_page_token,
                page_size=page_size,
                auth=auth,
            )
        )

    async def get_character_activity_access(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Retrieves the information for a Fireteam lobby.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/FireteamFinder/CharacterActivityAccess/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="GET",
                auth=auth,
            )
        )

    async def get_offer(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        offer_id: int,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Retrieves an offer to a Fireteam lobby.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            offer_id: The unique ID of the offer.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/FireteamFinder/Offer/{offer_id}/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="GET",
                auth=auth,
            )
        )

    async def get_lobby_offers(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        lobby_id: int,
        next_page_token: Optional[str] = None,
        page_size: Optional[int] = None,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Retrieves all offers relevant to a Fireteam lobby.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The unique ID of the lobby.
            next_page_token: An optional token from a previous response to fetch the next page of results.
            page_size: The maximum number of results to be returned with this page.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/FireteamFinder/Lobby/{lobby_id}/Offers/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="GET",
                next_page_token=next_page_token,
                page_size=page_size,
                auth=auth,
            )
        )

    async def host_lobby(
        self,
        max_player_count: int,
        online_players_only: bool,
        privacy_scope: Union[Any, int],
        scheduled_date_time: datetime,
        clan_id: int,
        listing_values: list[Any],
        activity_graph_hash: int,
        activity_hash: int,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Creates a new Fireteam lobby and Fireteam Finder listing.

        Args:
            max_player_count: _No description given by bungie._
            online_players_only: _No description given by bungie._
            privacy_scope: _No description given by bungie._
            scheduled_date_time: _No description given by bungie._
            clan_id: _No description given by bungie._
            listing_values: _No description given by bungie._
            activity_graph_hash: _No description given by bungie._
            activity_hash: _No description given by bungie._
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "maxPlayerCount": max_player_count,
            "onlinePlayersOnly": online_players_only,
            "privacyScope": privacy_scope,
            "scheduledDateTime": scheduled_date_time,
            "clanId": clan_id,
            "listingValues": listing_values,
            "activityGraphHash": activity_graph_hash,
            "activityHash": activity_hash,
        }

        return await self.request(
            Route(
                path=f"/FireteamFinder/Lobby/Host/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="POST",
                data=data,
                auth=auth,
            )
        )

    async def join_lobby(
        self,
        lobby_id: int,
        offer_id: int,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Sends a request to join an available Fireteam lobby.

        Args:
            lobby_id: _No description given by bungie._
            offer_id: _No description given by bungie._
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "lobbyId": lobby_id,
            "offerId": offer_id,
        }

        return await self.request(
            Route(
                path=f"/FireteamFinder/Lobby/Join/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="POST",
                data=data,
                auth=auth,
            )
        )

    async def kick_player(
        self,
        target_membership_type: Union[Any, int],
        target_character_id: int,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        lobby_id: int,
        target_membership_id: int,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Kicks a player from a Fireteam Finder lobby.

        Args:
            target_membership_type: _No description given by bungie._
            target_character_id: _No description given by bungie._
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The ID of the lobby to kick the player from.
            target_membership_id: A valid Destiny membership ID of the player to kick.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "targetMembershipType": target_membership_type,
            "targetCharacterId": target_character_id,
        }

        return await self.request(
            Route(
                path=f"/FireteamFinder/Lobby/{lobby_id}/KickPlayer/{target_membership_id}/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="POST",
                data=data,
                auth=auth,
            )
        )

    async def leave_application(
        self,
        application_id: int,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Sends a request to leave a Fireteam listing application.

        Args:
            application_id: The ID of the application to leave.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/FireteamFinder/Application/Leave/{application_id}/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="POST",
                auth=auth,
            )
        )

    async def leave_lobby(
        self,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        lobby_id: int,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Sends a request to leave a Fireteam lobby.

        Args:
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The ID of the lobby to leave.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/FireteamFinder/Lobby/Leave/{lobby_id}/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="POST",
                auth=auth,
            )
        )

    async def respond_to_application(
        self,
        accepted: bool,
        application_id: int,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Responds to an application sent to a Fireteam lobby.

        Args:
            accepted: _No description given by bungie._
            application_id: The application ID to respond to.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "accepted": accepted,
        }

        return await self.request(
            Route(
                path=f"/FireteamFinder/Application/Respond/{application_id}/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="POST",
                data=data,
                auth=auth,
            )
        )

    async def respond_to_authentication(
        self,
        confirmed: bool,
        application_id: int,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Responds to an authentication request for a Fireteam.

        Args:
            confirmed: _No description given by bungie._
            application_id: The ID of the application whose authentication to confirm.
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "confirmed": confirmed,
        }

        return await self.request(
            Route(
                path=f"/FireteamFinder/Authentication/Respond/{application_id}/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="POST",
                data=data,
                auth=auth,
            )
        )

    async def respond_to_offer(
        self,
        accepted: bool,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        offer_id: int,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Responds to a Fireteam lobby offer.

        Args:
            accepted: _No description given by bungie._
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            offer_id: The unique ID of the offer.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "accepted": accepted,
        }

        return await self.request(
            Route(
                path=f"/FireteamFinder/Offer/Respond/{offer_id}/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="POST",
                data=data,
                auth=auth,
            )
        )

    async def search_listings_by_clan(
        self,
        page_size: int,
        page_token: str,
        lobby_state: Union[Any, int],
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Returns search results for available Fireteams provided a clan.

        Args:
            page_size: _No description given by bungie._
            page_token: _No description given by bungie._
            lobby_state: _No description given by bungie._
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "pageSize": page_size,
            "pageToken": page_token,
            "lobbyState": lobby_state,
        }

        return await self.request(
            Route(
                path=f"/FireteamFinder/Search/Clan/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="POST",
                data=data,
                auth=auth,
            )
        )

    async def search_listings_by_filters(
        self,
        filters: list[Any],
        page_size: int,
        page_token: str,
        lobby_state: Union[Any, int],
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Returns search results for available Fireteams provided search filters.

        Args:
            filters: _No description given by bungie._
            page_size: _No description given by bungie._
            page_token: _No description given by bungie._
            lobby_state: _No description given by bungie._
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "filters": filters,
            "pageSize": page_size,
            "pageToken": page_token,
            "lobbyState": lobby_state,
        }

        return await self.request(
            Route(
                path=f"/FireteamFinder/Search/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="POST",
                data=data,
                auth=auth,
            )
        )

    async def update_lobby_settings(
        self,
        updated_settings: Any,
        destiny_character_id: int,
        destiny_membership_id: int,
        destiny_membership_type: int,
        lobby_id: int,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Updates the settings for a Fireteam lobby.

        Args:
            updated_settings: _No description given by bungie._
            destiny_character_id: A valid Destiny character ID.
            destiny_membership_id: A valid Destiny membership ID.
            destiny_membership_type: A valid Destiny membership type.
            lobby_id: The ID of the lobby to update.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "updatedSettings": updated_settings,
        }

        return await self.request(
            Route(
                path=f"/FireteamFinder/Lobby/UpdateSettings/{lobby_id}/{destiny_membership_type}/{destiny_membership_id}/{destiny_character_id}/",
                method="POST",
                data=data,
                auth=auth,
            )
        )



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from datetime import datetime
from typing import Optional, TYPE_CHECKING

from bungio.models.base import BaseModel, custom_define, custom_field


if TYPE_CHECKING:
    from bungio.models import DestinyVendorGroupDefinition
    from bungio.models import DestinyVendorDefinition
    from bungio.models import DestinyItemQuantity
    from bungio.models import DestinyInventoryItemDefinition


@custom_define()
class DestinyVendorGroupComponent(BaseModel):
    """
    This component returns references to all of the Vendors in the response, grouped by categorizations that Bungie has deemed to be interesting, in the order in which both the groups and the vendors within that group should be rendered.

    None
    Attributes:
        groups: The ordered list of groups being returned.
    """

    groups: list["DestinyVendorGroup"] = custom_field(metadata={"type": """list[DestinyVendorGroup]"""})


@custom_define()
class DestinyVendorGroup(BaseModel):
    """
    Represents a specific group of vendors that can be rendered in the recommended order. How do we figure out this order? It's a long story, and will likely get more complicated over time.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        vendor_group_hash: _No description given by bungie._
        vendor_hashes: The ordered list of vendors within a particular group.
        manifest_vendor_group_hash: Manifest information for `vendor_group_hash`
    """

    vendor_group_hash: int = custom_field()
    vendor_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    manifest_vendor_group_hash: Optional["DestinyVendorGroupDefinition"] = custom_field(default=None)


@custom_define()
class DestinyVendorBaseComponent(BaseModel):
    """
    This component contains essential/summary information about the vendor.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        enabled: If True, the Vendor is currently accessible.  If False, they may not actually be visible in the world at the moment.
        next_refresh_date: The date when this vendor's inventory will next rotate/refresh. Note that this is distinct from the date ranges that the vendor is visible/available in-game: this field indicates the specific time when the vendor's available items refresh and rotate, regardless of whether the vendor is actually available at that time. Unfortunately, these two values may be (and are, for the case of important vendors like Xur) different. Issue https://github.com/Bungie-net/api/issues/353 is tracking a fix to start providing visibility date ranges where possible in addition to this refresh date, so that all important dates for vendors are available for use.
        vendor_hash: The unique identifier for the vendor. Use it to look up their DestinyVendorDefinition.
        manifest_vendor_hash: Manifest information for `vendor_hash`
    """

    enabled: bool = custom_field()
    next_refresh_date: datetime = custom_field()
    vendor_hash: int = custom_field()
    manifest_vendor_hash: Optional["DestinyVendorDefinition"] = custom_field(default=None)


@custom_define()
class DestinyVendorSaleItemBaseComponent(BaseModel):
    """
    The base class for Vendor Sale Item data. Has a bunch of character-agnostic state about the item being sold. Note that if you want instance, stats, etc... data for the item, you'll have to request additional components such as ItemInstances, ItemPerks etc... and acquire them from the DestinyVendorResponse's "items" property.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        api_purchasable: If true, this item can be purchased through the Bungie.net API.
        costs: A summary of the current costs of the item.
        item_hash: The hash of the item being sold, as a quick shortcut for looking up the DestinyInventoryItemDefinition of the sale item.
        override_next_refresh_date: If this item has its own custom date where it may be removed from the Vendor's rotation, this is that date. Note that there's not actually any guarantee that it will go away: it could be chosen again and end up still being in the Vendor's sale items! But this is the next date where that test will occur, and is also the date that the game shows for availability on things like Bounties being sold. So it's the best we can give.
        override_style_item_hash: If populated, this is the hash of the item whose icon (and other secondary styles, but *not* the human readable strings) should override whatever icons/styles are on the item being sold. If you don't do this, certain items whose styles are being overridden by socketed items - such as the "Recycle Shader" item - would show whatever their default icon/style is, and it wouldn't be pretty or look accurate.
        quantity: How much of the item you'll be getting.
        vendor_item_index: The index into the DestinyVendorDefinition.itemList property. Note that this means Vendor data *is* Content Version dependent: make sure you have the latest content before you use Vendor data, or these indexes may mismatch.  Most systems avoid this problem, but Vendors is one area where we are unable to reasonably avoid content dependency at the moment.
        manifest_item_hash: Manifest information for `item_hash`
        manifest_override_style_item_hash: Manifest information for `override_style_item_hash`
    """

    api_purchasable: bool = custom_field()
    costs: list["DestinyItemQuantity"] = custom_field(metadata={"type": """list[DestinyItemQuantity]"""})
    item_hash: int = custom_field()
    override_next_refresh_date: datetime = custom_field()
    override_style_item_hash: int = custom_field()
    quantity: int = custom_field()
    vendor_item_index: int = custom_field()
    manifest_item_hash: Optional["DestinyInventoryItemDefinition"] = custom_field(default=None)
    manifest_override_style_item_hash: Optional["DestinyInventoryItemDefinition"] = custom_field(default=None)


@custom_define()
class DestinyPublicVendorComponent(BaseModel):
    """
    This component contains essential/summary information about the vendor from the perspective of a character-agnostic view.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        enabled: If True, the Vendor is currently accessible.  If False, they may not actually be visible in the world at the moment.
        next_refresh_date: The date when this vendor's inventory will next rotate/refresh. Note that this is distinct from the date ranges that the vendor is visible/available in-game: this field indicates the specific time when the vendor's available items refresh and rotate, regardless of whether the vendor is actually available at that time. Unfortunately, these two values may be (and are, for the case of important vendors like Xur) different. Issue https://github.com/Bungie-net/api/issues/353 is tracking a fix to start providing visibility date ranges where possible in addition to this refresh date, so that all important dates for vendors are available for use.
        vendor_hash: The unique identifier for the vendor. Use it to look up their DestinyVendorDefinition.
        manifest_vendor_hash: Manifest information for `vendor_hash`
    """

    enabled: bool = custom_field()
    next_refresh_date: datetime = custom_field()
    vendor_hash: int = custom_field()
    manifest_vendor_hash: Optional["DestinyVendorDefinition"] = custom_field(default=None)


@custom_define()
class DestinyPublicVendorSaleItemComponent(BaseModel):
    """
    Has character-agnostic information about an item being sold by a vendor. Note that if you want instance, stats, etc... data for the item, you'll have to request additional components such as ItemInstances, ItemPerks etc... and acquire them from the DestinyVendorResponse's "items" property. For most of these, however, you'll have to ask for it in context of a specific character.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        api_purchasable: If true, this item can be purchased through the Bungie.net API.
        costs: A summary of the current costs of the item.
        item_hash: The hash of the item being sold, as a quick shortcut for looking up the DestinyInventoryItemDefinition of the sale item.
        override_next_refresh_date: If this item has its own custom date where it may be removed from the Vendor's rotation, this is that date. Note that there's not actually any guarantee that it will go away: it could be chosen again and end up still being in the Vendor's sale items! But this is the next date where that test will occur, and is also the date that the game shows for availability on things like Bounties being sold. So it's the best we can give.
        override_style_item_hash: If populated, this is the hash of the item whose icon (and other secondary styles, but *not* the human readable strings) should override whatever icons/styles are on the item being sold. If you don't do this, certain items whose styles are being overridden by socketed items - such as the "Recycle Shader" item - would show whatever their default icon/style is, and it wouldn't be pretty or look accurate.
        quantity: How much of the item you'll be getting.
        vendor_item_index: The index into the DestinyVendorDefinition.itemList property. Note that this means Vendor data *is* Content Version dependent: make sure you have the latest content before you use Vendor data, or these indexes may mismatch.  Most systems avoid this problem, but Vendors is one area where we are unable to reasonably avoid content dependency at the moment.
        manifest_item_hash: Manifest information for `item_hash`
        manifest_override_style_item_hash: Manifest information for `override_style_item_hash`
    """

    api_purchasable: bool = custom_field()
    costs: list["DestinyItemQuantity"] = custom_field(metadata={"type": """list[DestinyItemQuantity]"""})
    item_hash: int = custom_field()
    override_next_refresh_date: datetime = custom_field()
    override_style_item_hash: int = custom_field()
    quantity: int = custom_field()
    vendor_item_index: int = custom_field()
    manifest_item_hash: Optional["DestinyInventoryItemDefinition"] = custom_field(default=None)
    manifest_override_style_item_hash: Optional["DestinyInventoryItemDefinition"] = custom_field(default=None)



from bungio.http.routes.app import AppRouteHttpRequests
from bungio.http.routes.user import UserRouteHttpRequests
from bungio.http.routes.content import ContentRouteHttpRequests
from bungio.http.routes.forum import ForumRouteHttpRequests
from bungio.http.routes.group_v2 import GroupV2RouteHttpRequests
from bungio.http.routes.tokens import TokensRouteHttpRequests
from bungio.http.routes.destiny2 import Destiny2RouteHttpRequests
from bungio.http.routes.community_content import CommunityContentRouteHttpRequests
from bungio.http.routes.trending import TrendingRouteHttpRequests
from bungio.http.routes.fireteam import FireteamRouteHttpRequests
from bungio.http.routes.fireteam_finder import FireteamFinderRouteHttpRequests
from bungio.http.routes.social import SocialRouteHttpRequests
from bungio.http.routes.get_available_locales import GetAvailableLocalesRouteHttpRequests
from bungio.http.routes.settings import SettingsRouteHttpRequests
from bungio.http.routes.user_system_overrides import UserSystemOverridesRouteHttpRequests
from bungio.http.routes.global_alerts import GlobalAlertsRouteHttpRequests


class AllRouteHttpRequests(
    AppRouteHttpRequests,
    UserRouteHttpRequests,
    ContentRouteHttpRequests,
    ForumRouteHttpRequests,
    GroupV2RouteHttpRequests,
    TokensRouteHttpRequests,
    Destiny2RouteHttpRequests,
    CommunityContentRouteHttpRequests,
    TrendingRouteHttpRequests,
    FireteamRouteHttpRequests,
    FireteamFinderRouteHttpRequests,
    SocialRouteHttpRequests,
    GetAvailableLocalesRouteHttpRequests,
    SettingsRouteHttpRequests,
    UserSystemOverridesRouteHttpRequests,
    GlobalAlertsRouteHttpRequests,
):
    pass



from typing import Callable, Coroutine, Optional

from bungio.http.route import Route
from bungio.models.auth import AuthData


class UserRouteHttpRequests:
    request: Callable[..., Coroutine]

    async def get_bungie_net_user_by_id(self, id: int, auth: Optional[AuthData] = None, *args, **kwargs) -> dict:
        """
        Loads a bungienet user by membership id.

        Args:
            id: The requested Bungie.net membership id.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(Route(path=f"/User/GetBungieNetUserById/{id}/", method="GET", auth=auth))

    async def get_sanitized_platform_display_names(
        self, membership_id: int, auth: Optional[AuthData] = None, *args, **kwargs
    ) -> dict:
        """
        Gets a list of all display names linked to this membership id but sanitized (profanity filtered). Obeys all visibility rules of calling user and is heavily cached.

        Args:
            membership_id: The requested membership id to load.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(path=f"/User/GetSanitizedPlatformDisplayNames/{membership_id}/", method="GET", auth=auth)
        )

    async def get_credential_types_for_target_account(
        self, membership_id: int, auth: Optional[AuthData] = None, *args, **kwargs
    ) -> dict:
        """
        Returns a list of credential types attached to the requested account

        Args:
            membership_id: The user's membership id
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(path=f"/User/GetCredentialTypesForTargetAccount/{membership_id}/", method="GET", auth=auth)
        )

    async def get_available_themes(self, auth: Optional[AuthData] = None, *args, **kwargs) -> dict:
        """
        Returns a list of all available user themes.

        Args:
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(Route(path="/User/GetAvailableThemes/", method="GET", auth=auth))

    async def get_membership_data_by_id(
        self, membership_id: int, membership_type: int, auth: Optional[AuthData] = None, *args, **kwargs
    ) -> dict:
        """
        Returns a list of accounts associated with the supplied membership ID and membership type. This will include all linked accounts (even when hidden) if supplied credentials permit it.

        Args:
            membership_id: The membership ID of the target user.
            membership_type: Type of the supplied membership ID.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(path=f"/User/GetMembershipsById/{membership_id}/{membership_type}/", method="GET", auth=auth)
        )

    async def get_membership_data_for_current_user(self, auth: AuthData, *args, **kwargs) -> dict:
        """
        Returns a list of accounts associated with signed in user. This is useful for OAuth implementations that do not give you access to the token response.

        Warning: Requires Authentication.
            Required oauth2 scopes: ReadBasicUserProfile

        Args:
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(Route(path="/User/GetMembershipsForCurrentUser/", method="GET", auth=auth))

    async def get_membership_from_hard_linked_credential(
        self, credential: str, cr_type: int, auth: Optional[AuthData] = None, *args, **kwargs
    ) -> dict:
        """
        Gets any hard linked membership given a credential. Only works for credentials that are public (just SteamID64 right now). Cross Save aware.

        Args:
            credential: The credential to look up. Must be a valid SteamID64.
            cr_type: The credential type. 'SteamId' is the only valid value at present.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(path=f"/User/GetMembershipFromHardLinkedCredential/{cr_type}/{credential}/", method="GET", auth=auth)
        )

    async def search_by_global_name_prefix(
        self, display_name_prefix: str, page: int, auth: Optional[AuthData] = None, *args, **kwargs
    ) -> dict:
        """
        [OBSOLETE] Do not use this to search users, use SearchByGlobalNamePost instead.

        Args:
            display_name_prefix: The display name prefix you're looking for.
            page: The zero-based page of results you desire.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(path=f"/User/Search/Prefix/{display_name_prefix}/{page}/", method="GET", auth=auth)
        )

    async def search_by_global_name_post(
        self, display_name_prefix: str, page: int, auth: Optional[AuthData] = None, *args, **kwargs
    ) -> dict:
        """
        Given the prefix of a global display name, returns all users who share that name.

        Args:
            display_name_prefix: _No description given by bungie._
            page: The zero-based page of results you desire.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "displayNamePrefix": display_name_prefix,
        }

        return await self.request(Route(path=f"/User/Search/GlobalName/{page}/", method="POST", data=data, auth=auth))



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import Union, TYPE_CHECKING

from bungio.utils import enum_converter
from bungio.models.base import HashObject, ManifestModel, custom_define, custom_field


if TYPE_CHECKING:
    from bungio.models import DestinyDisplayPropertiesDefinition
    from bungio.models import DestinyBreakerType


@custom_define()
class DestinyBreakerTypeDefinition(ManifestModel, HashObject):
    """
    _No description given by bungie._

    None
    Attributes:
        display_properties: _No description given by bungie._
        enum_value: We have an enumeration for Breaker types for quick reference. This is the current definition's breaker type enum value.
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        index: The index of the entity as it was found in the investment tables.
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
    """

    display_properties: "DestinyDisplayPropertiesDefinition" = custom_field()
    enum_value: Union["DestinyBreakerType", int] = custom_field(converter=enum_converter("DestinyBreakerType"))
    index: int = custom_field()
    redacted: bool = custom_field()



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.


from bungio.models.base import BaseModel, custom_define, custom_field


@custom_define()
class DestinyManifest(BaseModel):
    """
    DestinyManifest is the external-facing contract for just the properties needed by those calling the Destiny Platform.

    None
    Attributes:
        icon_image_pyramid_info: Information about the "Image Pyramid" for Destiny icons. Where possible, we create smaller versions of Destiny icons. These are found as subfolders under the location of the "original/full size" Destiny images, with the same file name and extension as the original image itself. (this lets us avoid sending largely redundant path info with every entity, at the expense of the smaller versions of the image being less discoverable)
        json_world_component_content_paths: This points to the generated JSON that contains all the Definitions. Each key is a locale. The value is a dictionary, where the key is a definition type by name, and the value is the path to the file for that definition. WARNING: This is unsafe and subject to change - do not depend on data in these files staying around long-term.
        json_world_content_paths: This points to the generated JSON that contains all the Definitions. Each key is a locale. The value is a path to the aggregated world definitions (warning: large file!)
        mobile_asset_content_path: _No description given by bungie._
        mobile_clan_banner_database_path: _No description given by bungie._
        mobile_gear_asset_data_bases: _No description given by bungie._
        mobile_gear_c_d_n: _No description given by bungie._
        mobile_world_content_paths: _No description given by bungie._
        version: _No description given by bungie._
    """

    icon_image_pyramid_info: list["ImagePyramidEntry"] = custom_field(metadata={"type": """list[ImagePyramidEntry]"""})
    json_world_component_content_paths: dict[str, dict[str, str]] = custom_field(
        metadata={"type": """dict[str, dict[str, str]]"""}
    )
    json_world_content_paths: dict[str, str] = custom_field(metadata={"type": """dict[str, str]"""})
    mobile_asset_content_path: str = custom_field()
    mobile_clan_banner_database_path: str = custom_field()
    mobile_gear_asset_data_bases: list["GearAssetDataBaseDefinition"] = custom_field(
        metadata={"type": """list[GearAssetDataBaseDefinition]"""}
    )
    mobile_gear_c_d_n: dict[str, str] = custom_field(metadata={"type": """dict[str, str]"""})
    mobile_world_content_paths: dict[str, str] = custom_field(metadata={"type": """dict[str, str]"""})
    version: str = custom_field()


@custom_define()
class GearAssetDataBaseDefinition(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        path: _No description given by bungie._
        version: _No description given by bungie._
    """

    path: str = custom_field()
    version: int = custom_field()


@custom_define()
class ImagePyramidEntry(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        factor: The factor by which the original image size has been reduced.
        name: The name of the subfolder where these images are located.
    """

    factor: float = custom_field()
    name: str = custom_field()



from typing import TYPE_CHECKING, Union

from bungio.models.base import MISSING, BaseModel, custom_define, custom_field
from bungio.utils import enum_converter

if TYPE_CHECKING:
    from bungio.models import GroupDateRange, GroupSortBy, GroupType


@custom_define()
class GroupQuery(BaseModel):
    """
    NOTE: GroupQuery, as of Destiny 2, has essentially two totally different and incompatible "modes". If you are querying for a group, you can pass any of the properties below. If you are querying for a Clan, you MUST NOT pass any of the following properties (they must be null or undefined in your request, not just empty string/default values): - groupMemberCountFilter - localeFilter - tagText If you pass these, you will get a useless InvalidParameters error.

    None
    Attributes:
        creation_date: _No description given by bungie._
        current_page: _No description given by bungie._
        group_member_count_filter: _No description given by bungie._
        group_type: _No description given by bungie._
        items_per_page: _No description given by bungie._
        locale_filter: _No description given by bungie._
        name: _No description given by bungie._
        request_continuation_token: _No description given by bungie._
        sort_by: _No description given by bungie._
        tag_text: _No description given by bungie._
    """

    creation_date: Union["GroupDateRange", int] = custom_field(
        converter=enum_converter("GroupDateRange"), metadata={"type": "GroupDateRange"}, default=MISSING
    )
    current_page: int = custom_field(default=MISSING)
    group_member_count_filter: int = custom_field(default=MISSING)
    group_type: Union["GroupType", int] = custom_field(
        converter=enum_converter("GroupType"), metadata={"type": "GroupType"}, default=MISSING
    )
    items_per_page: int = custom_field(default=MISSING)
    locale_filter: str = custom_field(default=MISSING)
    name: str = custom_field(default=MISSING)
    request_continuation_token: str = custom_field(default=MISSING)
    sort_by: Union["GroupSortBy", int] = custom_field(
        converter=enum_converter("GroupSortBy"), metadata={"type": "GroupSortBy"}, default=MISSING
    )
    tag_text: str = custom_field(default=MISSING)
